# -*- coding:utf-8 -*-
"""
Created on 2016-6-7
@author: WeiYanfeng

从 CHubCallbackBase 继承，借助 Queue 实现后台业务逻辑

"""
import _addpath
import sys
import gevent
from gevent.queue import Queue
from weberFuncs import GetCurrentTime,PrintTimeMsg,PrintAndSleep,ClassForAttachAttr
from cpyf.cstpFuncs import GetCmdReplyFmRequest
from cpyf.mGlobalConst import CMD0_ECHO_CMD

from hub.CHubCallbackBasicBase import CHubCallbackBasicBase


class CHubCallbackQueueBase(CHubCallbackBasicBase):
    def __init__(self,sHubId):
        CHubCallbackBasicBase.__init__(self,sHubId)
        self.__queue4Return = Queue()  # 当前应答队列

    def PutCmdStrToReturnQueue(self, lsCmdStr):
        self.__queue4Return.put(lsCmdStr)

    def GetCmdStrFmReturnQueue(self):
        return self.__queue4Return.get()  # (sClientIPPort,dwCmdId,CmdOStr)

    def HandleRequestCmd(self, sClientIPPort, dwCmdId, CmdIStr):
        # 处理客户端请求命令
        bDone = CHubCallbackBasicBase.HandleRequestCmd(self, sClientIPPort, dwCmdId, CmdIStr)
        if not bDone and CmdIStr[0].startswith(CMD0_ECHO_CMD):
            CmdOStr = ['OK', 'CHubCallbackQueueBase']
            CmdOStr.extend(CmdIStr)
            dwCmdId = GetCmdReplyFmRequest(dwCmdId)
            self.PutCmdStrToReturnQueue([sClientIPPort ,dwCmdId, CmdOStr])
            bDone = True
        return bDone

    def DoHandleCheckAllLinkReply(self):
        # 处理检查所有链接的应答返还消息（包括通知消息）等
        # 该函数在该类中实现后，一般情况下子类无需再继承。
        while not self.bQuitLoopFlag:
            return self.GetCmdStrFmReturnQueue()


def tryCHubCallbackQueueBase():
    bhc = CHubCallbackQueueBase('sHubId')  # oObj

if __name__ == '__main__':
    tryCHubCallbackQueueBase()


#!/usr/local/bin/python
# -*- coding:utf-8 -*-
"""    
Created on 2016-7-2
@author: WeiYanfeng

    P2PLayout 模式演示用的参数配置，由 CHubCallbackP2PLayout.py 使用。
"""

gDictP2PLayoutByPairId = {
    # P2PLayout sPairId配置参数
    # 以 sPairId 为键值，内容是 sSuffix = sPassword 组成的键值对
    '@sPairId': {
        '@sSuffix': '@sPassword',
    },
    'one': {
        'A': 'onePairA',
        'B': 'onePairB',
        'C': 'onePairC',
        'D': 'onePairD',
        'Test': 'onePairTest',
    },
}



#!/usr/local/bin/python
# -*- coding:utf-8 -*-
"""
    2016/8/26  WeiYanfeng
    实现了 TCmdPipeServer 和 P2PLayout模式下Peer端的对接。
"""
import _addpath
import sys
from weberFuncs import PrintTimeMsg
from peer.TCmdStringSckP2PLayout import TCmdStringSckP2PLayout
from pipe.TCmdPipeServerTCBQ import TCmdPipeServerTCBQ


class TCmdStringSckP2PLayoutPipe(TCmdStringSckP2PLayout):
    def __init__(self, sHubId, sHostAndPort, sPairId, sSuffix, sAcctPwd,
                 sClientInfo, iPrintIntervalNum, iPipeServerPort, sPipeServerIP='127.0.0.1'):
        self.iPrintIntervalNum = iPrintIntervalNum  # 打印间隔数字, 越大日志越少
        self.bVerbose = self.iPrintIntervalNum <= 0  # 全部打印;
        TCmdStringSckP2PLayout.__init__(self, sHubId, sHostAndPort, sPairId, sSuffix, sAcctPwd,
                                        sClientInfo, self.bVerbose)
        PrintTimeMsg('TCmdStringSckP2PLayoutPipe.PipeServerIPPort=(%s:%s)' % (sPipeServerIP, iPipeServerPort))
        self.pipeServer = TCmdPipeServerTCBQ(iPipeServerPort, sPipeServerIP, self.HandlePipePushData)

    def HandlePipePushData(self, oData, iDealCount):
        if self.bVerbose:
            PrintTimeMsg('TCmdStringSckP2PLayoutPipe.HandlePipePushData.%d#.oData=%s=' % (iDealCount, oData))
        else:
            if iDealCount % self.iPrintIntervalNum == 0:
                PrintTimeMsg('PipePushData#%d.oData=%s=' % (iDealCount, ','.join(oData)))
        sRcv = '*'
        lsParam = [str(oData)]
        if type(oData) == list:
            sRcv = oData[0]
            lsParam = oData[1:]
        self.SendRequestP2PLayoutCmd(sRcv, lsParam, 'sLogicParamPipe')
        # WeiYF.20160831 发送日志如下：
        # [2016-08-31 16:25:27.342]SerialCstCmdStrToString.dwCmdId=1978.CmdCnt=8={
        #   CmdStr[0].24=!P2PLayout.SendCmdToPeer=
        #   CmdStr[1].14=SndMinK.FeedIQ=
        #   CmdStr[2].9=RcvMinK.*=
        #   CmdStr[3].18=!MinuteK.PushRedis=
        #   CmdStr[4].9=kline_chg=
        #   CmdStr[5].13=XTIUSD_201610=
        #   CmdStr[6].1=N=
        #   CmdStr[7].77=20160831-162500.000554,46.07,46.07,46.05,46.07,95,81,C,20160831-162523.452846=
        # }
        # 可以看出，CmdStr[0]是固定命令，CmdStr[1]是发送者的sSuffix
        # CmdStr[2+]才是 TCmdPipeClient 填入的参数。

    def StartMainThreadLoop(self):
        TCmdStringSckP2PLayout.StartMainThreadLoop(self)
        # WeiYF.20160901 主循环退出时，使 pipeServer 也退出
        self.pipeServer.SetLoopRunFlagToQuit('StartMainThreadLoop.End!')



# -*- coding:utf-8 -*-
"""
Created on 2016-6-30
@author: WeiYanfeng

从 TCmdStringSck 继承，实现 AcctShare 的客户端。

"""
import _addpath
import sys
from weberFuncs import PrintTimeMsg, GetCurrentTime, PrintAndSleep
from cpyf.mGlobalConst import CMD0_ECHO_CMD, P2PKIND_ACCTSHARE
from peer.TCmdStringSck import TCmdStringSck


class TCmdStringSckAcctShare(TCmdStringSck):
    def __init__(self, sHostAndPort, sAcctId, sAcctPwd, sClientInfo, bVerbosePrintCmdStr=True):
        TCmdStringSck.__init__(self, '@HubId', P2PKIND_ACCTSHARE, sHostAndPort,
                               sAcctId, sAcctPwd, 'Y', sClientInfo, bVerbosePrintCmdStr)

    def LoopAndProcessLogic(self):
        iCnt = 0
        while True:
            sLogicParam = 'LogParam'+str(iCnt)
            self.SendRequestCmd((CMD0_ECHO_CMD, 'Just4Test', "python"+str(iCnt), sLogicParam), sLogicParam)
            iCnt += 1
            if iCnt == 5:
                self.SendNotifyMsg(("LmtApi.NotifyMsg", 'Just4Test', "python"+str(iCnt), sLogicParam))
            if iCnt >= 10:
                break  # 10
            # time.sleep(0.1)

    def OnHandleReplyCallBack(self, sCmd0, sLogicParam, CmdStr, dwCmdId):
        PrintTimeMsg("TCmdStringSckAppShr.OnHandleReplyCallBack.sCmd0=%s,dwCmdId=%s,sLogicParam=%s,CmdStr=%s"
                    % (sCmd0, dwCmdId, sLogicParam, str(CmdStr)))
        return True


def TryTCmdStringSckAcctShare():
    cssa = TCmdStringSckAcctShare("127.0.0.1:8888", 'testCSTP', 'testCSTP', 'sClientDevInfo')
    cssa.StartMainThreadLoop()

# -------------------------------------
if __name__ == '__main__':
    TryTCmdStringSckAcctShare()



#!/usr/local/bin/python
# -*- coding:utf-8 -*-
"""    
    2015/10/14  WeiYanfeng
    公共函数 包
"""
# WeiYF.20170605 经测试，Python3引用当前目录下的源码文件，文件名前要加`.`点号
# 同时这样的写法也可以在Python2.7下使用。

from cpyf.cstpFuncs import SerialCmdStrToString, SerialCmdStrFmString

from hub.TCmdStringHub import TCmdStringHub, StartCmdStringHub
from hub.CHubCallbackBasicBase import CHubCallbackBasicBase
from hub.CHubCallbackQueueBase import CHubCallbackQueueBase
from hub.CHubCallbackP2PLayout import CHubCallbackP2PLayout

from peer.TCmdStringSck import TCmdStringSck, CssException
from peer.TCmdStringSckAcctShare import TCmdStringSckAcctShare
from peer.TCmdStringSckP2PLayout import TCmdStringSckP2PLayout, \
    StartCmdStringSckP2PLayout, StartCmdStringSckP2PLayoutSH

from pipe.TCmdPipeClient import TCmdPipeClient
from pipe.TCmdPipeServer import TCmdPipeServer
from pipe.TCmdPipeServerTCBQ import TCmdPipeServerTCBQ
from pipe.TCmdStringSckP2PLayoutPipe import TCmdStringSckP2PLayoutPipe

if __name__ == '__main__':
    pass



#!/usr/local/bin/python
# -*- coding:utf-8 -*-
"""    
    2015-04-13  WeiYanfeng
    该单元会自动将上级目录加入到 sys.path 方便于模块间相互引用
"""


def GetThisFilePath(sSubDir=".", sFName=""):
    import os, sys, inspect
    cmd_subfolder = os.path.realpath(os.path.abspath(os.path.join(
        os.path.split(inspect.getfile(inspect.currentframe()))[0], sSubDir)))
    if sFName:
        return cmd_subfolder+os.sep+sFName
    return cmd_subfolder


def AddThisFilePath(sSubDir):
    import sys
    cmd_subfolder = GetThisFilePath(sSubDir)
    # print("_init_.cmd_subfolder",cmd_subfolder)
    if cmd_subfolder not in sys.path:
        sys.path.insert(0, cmd_subfolder)


AddThisFilePath('..')



#!/usr/local/bin/python
# -*- coding:utf-8 -*-
"""
    2016/8/5  WeiYanfeng
    基于串列表的命名管道（借助 socket 实现）客户端。

"""
import _addpath
import sys
import os
import socket
from weberFuncs import GetCurrentTime, PrintTimeMsg, PrintAndSleep, GetCurrentTimeMSs
from cpyf.CSockReadWrite import CSockReadWrite


# -------------------------------------
class TCmdPipeClient:
    def __init__(self, sServerIPPort, sFileName4SE=''):
        # sFileName4SE = 全路径的文件名，用于对应服务端出现问题
        # WeiYF.20160805 sFileName4SE 默认取值为空串，表示无需输出
        self.sFileName4SE = sFileName4SE
        self.sServerIP = '127.0.0.1'
        self.iServerPort = 8888
        lsServer = sServerIPPort.split(':')
        if len(lsServer) >= 2:
            if lsServer[0]:
                self.sServerIP = lsServer[0]
            self.iServerPort = int(lsServer[1])
        self.sServerIPPort = '%s:%s' % (self.sServerIP, self.iServerPort)
        self.sockRW = None
        self.clientsocket = None

    def __del__(self):
        pass

    def ConnectToServer(self):
        self.clientsocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.clientsocket.connect((self.sServerIP,self.iServerPort))
        self.sockRW = CSockReadWrite(self.clientsocket,'C')
        self.sockRW.bVerbosePrintCmdStr = False
        self.sockRW.SetObjIPPort(self.sServerIPPort)
        PrintTimeMsg("ConnectToServer(%s).pid=(%s)" % (self.sServerIPPort,os.getpid()))

    def CloseSocket(self):
        if self.sockRW:
            self.sockRW.sock.close()
        PrintTimeMsg("CloseSocket(%s).pid=(%s)" % (self.sServerIPPort,os.getpid()))
        self.sockRW = None

    def SendPipeRequest(self, CmdStr):
        try:
            if self.sockRW == None:
                self.ConnectToServer()  # 放到try内部，避免出异常
            dwCmdId = 1225  # WeiYF.20160805 固定取值
            bResult = self.sockRW.WriteCmdStrToLink(dwCmdId,CmdStr)
            if bResult:
                return bResult
        except Exception as e:
            import traceback
            traceback.print_exc()
            PrintTimeMsg('SendPipeRequest.Exception={%s}!' % (str(e)))
        self.CloseSocket()
        self.AppendToFile4SE('|'.join(CmdStr))
        return False

    def AppendToFile4SE(self, sMsg):
        if self.sFileName4SE == '':
            return
        with open(self.sFileName4SE, "a") as f:  # 追加模式输出
            sS = "[%s]%s\n" % (GetCurrentTimeMSs(), sMsg)
            f.write(sS)


def tryTCmdPipeClient():
    # c = TCmdPipeClient('127.0.0.1:8805')
    # c = TCmdPipeClient('127.0.0.1:9801')
    c = TCmdPipeClient('192.168.2.199:9801')
    iCnt = 0
    while iCnt<5:
        # bRet = c.SendPipeRequest(['Test','One','2','three','iCnt=%d' % iCnt])
        bRet = c.SendPipeRequest(['*', 'Test', 'One', '2', 'three', 'iCnt=%d' % iCnt])
        if not bRet:
            PrintTimeMsg("testTCmdPipeClient.SendPipeRequest.Error.iCnt=%d" % iCnt)
            break
        PrintAndSleep(6, 'testTCmdPipeClient.iCnt=%d!' % iCnt)
        iCnt += 1

# -------------------------------------
if __name__ == '__main__':
    tryTCmdPipeClient()



# -*- coding:utf-8 -*-
"""
Created on 2016-6-30
@author: WeiYanfeng

从 TCmdStringSck 继承，实现 P2P11 的客户端。

"""
import _addpath
import sys
from weberFuncs import PrintTimeMsg, GetCurrentTime, PrintAndSleep, printCmdString
from cpyf.mGlobalConst import CMD0_ECHO_CMD, P2PKIND_P2PLAYOUT, CHAR_SEP_P2PLAYOUT
from cpyf.mP2PLayoutConst import CMD0_P2PLAYOUT_SEND_SYSTEM_MSG, CMD0_P2PLAYOUT_SEND_CMD_TOPEER
from peer.TCmdStringSck import TCmdStringSck

# ------------------------------------------------
# 参见 mP2PLayoutConst.py 说明：
# 基于 CMD0_P2PLAYOUT_SEND_CMD_TOPEER 遍历报数请求
CMD3_P2PLAYOUT_TRAVERSE_COUNT = 'TraverseCount'
#  4=iCountValueNow  # 当前报数结果
#  5=sSuffixDoneList # 已经完成报数的 sSuffix 列表串，逗号分隔
# ------------------------------------------------


class TCmdStringSckP2PLayout(TCmdStringSck):
    def __init__(self, sHubId, sHostAndPort, sPairId, sSuffix, sAcctPwd,
                 sClientInfo, bVerbosePrintCmdStr=True):
        self.sPairId = sPairId
        self.sSuffix = sSuffix
        self.sSuffixOnlineList = ''  # 当前在线后缀列表
        sAcctId = sPairId + CHAR_SEP_P2PLAYOUT + sSuffix
        TCmdStringSck.__init__(self, sHubId, P2PKIND_P2PLAYOUT, sHostAndPort,
                               sAcctId, sAcctPwd, 'Y',
                               sClientInfo, bVerbosePrintCmdStr)
        self.iOneSecondClock = 0
        self.sSuffixDoneList = ''

    def LoopAndProcessLogic(self):
        while not self.gef.IsExitFlagTrue():
            self.DoLoopOneSecondLogic()

    def DoLoopOneSecondLogic(self):
        self.iOneSecondClock += 1
        PrintAndSleep(1, 'DoLoopOneSecondLogic.iOneSecondClock=%d' % self.iOneSecondClock,
                      self.iOneSecondClock % 600 == 1)  # 10分钟打印一次日志
        if self.sSuffix == 'Test':
            self.TestLaunchTraverseCount(self.iOneSecondClock)
        pass

    def TestLaunchTraverseCount(self, iLoopCnt):
        self.LaunchTraverseCount()
        sLogicParam = 'LogParam@' + str(iLoopCnt)
        self.SendRequestP2PLayoutCmd('B*,A,C,D', ['TellMeCmd2', 'Just4TestFromA', "python"+str(iLoopCnt)], sLogicParam)
        self.SendRequestP2PLayoutCmd('A', ['SendNotifyMail', 'weiyf1225@qq.com',
                                           '标题title', '内容Content', 'sFromTitle测试'], sLogicParam)

    def OnHandleReplyCallBack(self, sCmd0, sLogicParam, CmdStr, dwCmdId):
        # WeiYF.20150715 为避免日志过多，暂时不打印
        # PrintTimeMsg("TCmdStringSckP2PLayout.OnHandleReplyCallBack.sCmd0=%s,dwCmdId=%s,sLogicParam=%s,CmdStr=%s"
        #             % (sCmd0, dwCmdId, sLogicParam, str(CmdStr)) )
        return True

    def OnHandleNotifyCallBack(self, CmdStr, dwCmdId):
        # 供子类继承，用于接收并处理通知消息
        if CmdStr[0] == CMD0_P2PLAYOUT_SEND_SYSTEM_MSG:  # 系统消息
            self.HandleSendSystemMsg(CmdStr)
        elif CmdStr[0] == CMD0_P2PLAYOUT_SEND_CMD_TOPEER:  # 预分配消息
            self.HandleSendCmdToPeer(CmdStr)
        else:
            TCmdStringSck.OnHandleNotifyCallBack(self, CmdStr, dwCmdId)

    def HandleSendSystemMsg(self, CmdStr):
        self.sSuffixOnlineList = CmdStr[4]
        # PrintTimeMsg("HandleSendSystemMsg.CmdStr=%s=" % (','.join(CmdStr)) )

    def SendRequestP2PLayoutCmd(self, sSuffixTarget, CmdStr, sLogicParam):
        # 发送P2PLayout请求命令，格式参见 mP2PLayoutConst.py
        CmdIStr = [CMD0_P2PLAYOUT_SEND_CMD_TOPEER, self.sSuffix, sSuffixTarget]
        CmdIStr.extend(CmdStr)
        # printCmdString('TCmdStringSckP2PLayout.SendRequestP2PLayoutCmd=',CmdIStr)
        # WeiYF.20160715 为避免日志过多
        self.SendRequestCmd(CmdIStr, sLogicParam)

    def HandleSendCmdToPeer(self, CmdStr):
        sSuffixFm = CmdStr[1]
        sSuffixTo = CmdStr[2]
        sPeerCmd = CmdStr[3]
        if sPeerCmd == CMD3_P2PLAYOUT_TRAVERSE_COUNT:
            self.DoTraverseCount(sSuffixFm, sSuffixTo, sPeerCmd, CmdStr)
        else:
            self.DoHandleSendCmdToPeer(sSuffixFm, sSuffixTo, sPeerCmd, CmdStr)

    def DoHandleSendCmdToPeer(self, sSuffixFm, sSuffixTo, sPeerCmd, CmdStr):
        # 返回 True 表示已经处理过
        PrintTimeMsg("DoHandleSendCmdToPeer.CmdStr=%s=" % (','.join(CmdStr)) )
        return False

    def DoTraverseCount(self, sSuffixFm, sSuffixTo, sPeerCmd, CmdStr):
        iCountValueNow = int(CmdStr[4])
        self.sSuffixDoneList = CmdStr[5]
        lsSuffixDone = self.sSuffixDoneList.split(',')
        lsSuffixOnline = self.sSuffixOnlineList.split(',')
        lsSuffixDone.append(self.sSuffix)
        lsLeft = []
        for sS in lsSuffixOnline:
            if sS not in lsSuffixDone:
                lsLeft.append(sS)
        if len(lsLeft) <= 0:
            PrintTimeMsg('DoTraverseCount.END!!!')
        else:
            iCountValueNow += 1
            sSuffixTarget = lsLeft[0]
            tupleParam = (CMD3_P2PLAYOUT_TRAVERSE_COUNT, str(iCountValueNow), ','.join(lsSuffixDone),)
            self.SendRequestP2PLayoutCmd(sSuffixTarget, tupleParam, 'sLogicParam')

    def LaunchTraverseCount(self):
        lsSuffixOnline = self.sSuffixOnlineList.split(',')
        lsLeft = []
        for sS in lsSuffixOnline:
            if sS != self.sSuffix:
                lsLeft.append(sS)
        if len(lsLeft) <= 0:
            PrintTimeMsg('LaunchTraverseCount.lsLeft=(%s)!' % ','.join(lsLeft))
        else:
            PrintTimeMsg('LaunchTraverseCount.lsLeft=(%s)!' % ','.join(lsLeft))
            sSuffixTarget = lsLeft[0]
            iCountValueNow = 1
            tupleParam = (CMD3_P2PLAYOUT_TRAVERSE_COUNT, str(iCountValueNow), self.sSuffix,)
            self.SendRequestP2PLayoutCmd(sSuffixTarget, tupleParam, 'sLogicParam')

# -------------------------------------


def TryTCmdStringSckP2PLayout():
    sHubId = 'fba008448317ea7f5c31f8e19c68fcf7'
    cssa = TCmdStringSckP2PLayout(sHubId,"127.0.0.1:8888",'one','Test','onePairTest','sClientDevInfo')
    cssa.StartMainThreadLoop()

# -------------------------------------
# WeiYF.20160708 将启动过程封装为一个共享函数，其中 dictPeerByParam 格式如下：
# gDictPeerByParam = {
#    # CSTP Peer运行参数，以 sHostName4Param 为键值
#     'LocalTest': {  #本地环境  #sHostName4Param
#         'sIPPort': '127.0.0.1:8888',
#         'sHubId': 'fba008448317ea7f5c31f8e19c68fcf7',
#         'sPairId': 'one',
#         'Wait2SendMail' :{  # sAppId
#             'sSuffix': 'A',
#             'sAcctPwd': 'onePairA',
#             'sClientInfo': 'sClientDevInfo',
#         },
#     },
# }
#


def StartCmdStringSckP2PLayout(dictPeerByParam, sHostName4Param, sAppId, clsSckP2PLayout):
    dictParam = dictPeerByParam.get(sHostName4Param, {})
    if not dictParam:
        PrintTimeMsg('StartCmdStringSckP2PLayout.get(%s)={}' % sHostName4Param)
        sys.exit(-1)
    dictApp = dictParam.get(sAppId, {})
    if not dictApp:
        PrintTimeMsg('StartCmdStringSckP2PLayout.get(%s)From(%s)={}' % (sAppId, str(dictParam)))
        sys.exit(-1)
    sIPPort = dictParam.get('sIPPort', '')
    sHubId = dictParam.get('sHubId', '')
    sPairId = dictParam.get('sPairId', '')
    sSuffix = dictApp.get('sSuffix', '')
    sAcctPwd = dictApp.get('sAcctPwd', '')
    sClientInfo = dictApp.get('sClientInfo', '')
    bVerbose = dictApp.get('bVerbose', 'True') == 'True'
    if sIPPort and sHubId and sPairId and sSuffix:
        cssa = clsSckP2PLayout(sHubId, sIPPort, sPairId, sSuffix, sAcctPwd, sClientInfo, bVerbose)
        cssa.StartMainThreadLoop()
    else:
        sErrMsg = ''
        if not sIPPort: sErrMsg += 'sIPPort,'
        if not sHubId: sErrMsg += 'sHubId,'
        if not sPairId: sErrMsg += 'sPairId,'
        if not sSuffix: sErrMsg += 'sSuffix,'
        sErrMsg = '(%s) is null' % sErrMsg[:-1]  # 剔除最后一个逗号
        PrintTimeMsg('StartCmdStringSckP2PLayout.sErrMsg=%s!' % sErrMsg)
        sys.exit(-1)

# -------------------------------------
# WeiYF.20160825 将启动过程封装为一个共享函数，其中 dictPeerByParam 格式如下：
gDictPeerByParam = {
    # CSTP Peer运行参数，以 sHostName4Param 为键值
    'LocalTest': {  # 本地环境  #sHostName4Param
        'sIPPort': '127.0.0.1:8888',
        'sHubId': 'fba008448317ea7f5c31f8e19c68fcf7',
        'sPairId': '@sPairId',
    },
    # WeiYF.20160825 Peer配置尽可能与Hub一样的结构，这样方便复制
    'SH_@sPairId': {
        '@sSuffix': '@sPassword',
    },
}


def StartCmdStringSckP2PLayoutSH(dictPeerByParam, sHostName4Param, sSuffix, bVerbose,
                                 clsSckP2PLayout, *args, **kwargs):
    # 创建并启动 clsSckP2PLayout 类
    # 出错后不要退出，改为返回 False ，交外围程序继续处理
    dictParam = dictPeerByParam.get(sHostName4Param, {})
    if not dictParam:
        PrintTimeMsg('StartCmdStringSckP2PLayoutSH.get(%s)={}Error!' % sHostName4Param)
        return False
    sPairId = dictParam.get('sPairId', '')
    if not sPairId:
        PrintTimeMsg('StartCmdStringSckP2PLayoutSH.sPairId=Null,Error!' % sPairId)
        return False
    sPairIdSH = 'SH_'+sPairId
    dictApp = dictPeerByParam.get(sPairIdSH, {})
    if not dictApp:
        PrintTimeMsg('StartCmdStringSckP2PLayoutSH.get(%s)={}Error!' % sPairIdSH)
        return False
    sIPPort = dictParam.get('sIPPort', '')
    sHubId = dictParam.get('sHubId', '')
    sAcctPwd = dictApp.get(sSuffix, '')
    if sIPPort and sHubId and sSuffix and sAcctPwd:
        sClientInfo = 'ShareHub'
        cssa = clsSckP2PLayout(sHubId, sIPPort, sPairId, sSuffix, sAcctPwd, sClientInfo,
                               bVerbose, *args, **kwargs)
        cssa.StartMainThreadLoop()
        return True
    else:
        sErrMsg = 'sSuffix=%s,' % sSuffix  # ''
        if not sIPPort:
            sErrMsg += 'sIPPort,'
        if not sHubId:
            sErrMsg += 'sHubId,'
        if not sSuffix:
            sErrMsg += 'sSuffix,'
        if not sAcctPwd:
            sErrMsg += 'sAcctPwd,'
        sErrMsg = '(%s) is null' % sErrMsg[:-1]  # 剔除最后一个逗号
        PrintTimeMsg('StartCmdStringSckP2PLayoutSH.sErrMsg=%s!Error!' % sErrMsg)
        return False


# -------------------------------------
if __name__ == '__main__':
    TryTCmdStringSckP2PLayout()



#!/usr/local/bin/python
# -*- coding:utf-8 -*-
"""
    2016/8/5  WeiYanfeng
    基于串列表的命名管道（借助 socket 实现）服务端。

"""
import _addpath
import sys
import os
import socket
from weberFuncs import GetCurrentTime, PrintTimeMsg, PrintAndSleep
from cpyf.CSockReadWrite import CSockReadWrite
from cpyf.CssException import CssException, gef


# -------------------------------------
class TCmdPipeServer:
    def __init__(self, iServerPort, sServerIP='0.0.0.0'):
        self.sServerIP = sServerIP
        self.iServerPort = iServerPort
        self.sServerIPPort = '%s:%s' % (self.sServerIP, self.iServerPort)

        self.serversocket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.serversocket.bind((self.sServerIP, self.iServerPort))
        self.serversocket.listen(5)  # become a server socket, maximum 5 connections
        # WeiYF.20160805 暂时保留5，看实际运行情况

        self.iSecondsTimeOut = 60  # 3600 #86400 # 读取超时
        self.bLoopRunFlag = True  # 循环判断变量

        self.sockRW = None

        PrintTimeMsg('TCmdPipeServer(%s).pid=(%s)Start...' % (
            self.sServerIPPort, os.getpid()))
        self.LoopAndWaitPipe()

    def __del__(self):
        pass

    def SetLoopRunFlagToQuit(self, sHint):
        self.bLoopRunFlag = False
        PrintTimeMsg("SetLoopRunFlagToQuit.sHint=%s=" % sHint)

    def LoopAndWaitPipe(self):
        iLoopCntAccept = 0
        while self.bLoopRunFlag and (not gef.IsExitFlagTrue()):
            self.AcceptOneClient()
            iLoopCntRead = 0
            while self.bLoopRunFlag and (not gef.IsExitFlagTrue()):
                try:
                    sRet, sMsg = self.sockRW.ReadCmdStrFromLink(self.iSecondsTimeOut)
                    if sRet == 'OK':
                        (dwCmdId, CmdStr) = sMsg
                        self.HandlePipeRequest(CmdStr)
                    else:
                        PrintTimeMsg("LoopAndWaitPipe.Error=%s,%s!" % (sRet, sMsg))
                        self.HandlePipeError(sRet, sMsg)
                        break
                except Exception as e:
                    import traceback
                    traceback.print_exc()  # WeiYF.20151022 打印异常整个堆栈 这个对于动态加载非常有用
                    gef.SetExitFlagTrue('LoopAndWaitPipe.e=%s=' % e)
                iLoopCntRead += 1
            self.sockRW.sock.close()
            iLoopCntAccept += 1
            PrintAndSleep(10, "LoopAndWaitPipe.iLoopCntAccept=%s,iLoopCntRead=%s" % (
                iLoopCntAccept, iLoopCntRead))
        PrintTimeMsg("LoopAndWaitPipe.iLoopCntAccept=%s,QUIT!" % iLoopCntAccept)

    def AcceptOneClient(self):
        connection, address = self.serversocket.accept()
        sClientIPPort = '%s:%s' % (address[0],address[1])
        PrintTimeMsg("AcceptOneClient.sClientIPPort=%s!" % sClientIPPort)
        self.sockRW = CSockReadWrite(connection,'S')
        self.sockRW.bVerbosePrintCmdStr = False
        self.sockRW.SetObjIPPort(sClientIPPort)
        self.sockRW.cLoginStatus = 'R'  # 默认就是运行状态

    def HandlePipeRequest(self, CmdStr):
        pass

    def HandlePipeError(self, sRet, sMsg):
        pass


def mainCmdPipeServer():
    iServerPort = 8805
    sServerIP = '0.0.0.0'
    c = TCmdPipeServer(iServerPort, sServerIP)
    pass

# -------------------------------------
if __name__ == '__main__':
    mainCmdPipeServer()



# -*- coding:utf-8 -*-
"""
Created on 2016-7-5
@author: WeiYanfeng

测试 TCmdStringSckP2PLayout 类。

"""
import _addpath
from weberFuncs import PrintTimeMsg, GetCurrentTime, PrintAndSleep

from peer.TCmdStringSckP2PLayout import TCmdStringSckP2PLayout


def TryTCmdStringSckP2PLayout():
    sHubId = 'fba008448317ea7f5c31f8e19c68fcf7'
    cssa = TCmdStringSckP2PLayout(sHubId, "127.0.0.1:8888", 'one', 'C', 'onePairC', 'sClientDevInfo')
    cssa.StartMainThreadLoop()

# -------------------------------------
if __name__ == '__main__':
    TryTCmdStringSckP2PLayout()
